using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Foo.Settings;

namespace zEasyTest
{
    static class Settings_Test
    {

        public static void MockAroundWithSettings()
        {
            OoganizerSettingsAcc acc = new OoganizerSettingsAcc();

            configuration conf = new configuration();
            conf.ButtonHook.AllowedProcessNames.AddProcessName(new ButtonHookSettings.ProcessName("Doris", "Doris1", "Doris3", "Doris4", "123", "dd", "vv", 0, 0));
            conf.ButtonHook.AllowedProcessNames.AddProcessName(new ButtonHookSettings.ProcessName("Voris", "Voris1", "Voris3", "Voris4", "123", "dd", "vv", 0, 0));
            conf.ButtonHook.AllowedProcessNames.AddProcessName(new ButtonHookSettings.ProcessName("Koris", "Koris1", "Koris3", "Koris4", "123", "dd", "vv", 0, 0));

            //configuration config = acc.ReadConfigXMLFile();

            //configuration config = acc.ReadConfigXMLFile(@"D:\Users\HPinvent\_ROOT_\TestANizer\DataAccessLayer\Doris.xml", false);            

            acc.WriteConfigXMLFile(@"D:\Users\HPinvent\_ROOT_\TestANizer\DataAccessLayer\Doris.xml", conf);

            //acc.WriteBlankConfigXMLFile(@"D:\Users\HPinvent\_ROOT_\TestANizer\DataAccessLayer\Doris.xml");
        }


    }
}
